var Fis75000_Efd_Reinf_Service = function(General, Messages_fiscal,ScreenService) {
	
	// Operações do EFD-REINF
    this.IDTRANSENVIO = 'ENVIAR'; //Enviar Evento
    this.IDTRANSCONSU = 'CONSULTAR'; //Consultar Status do evento
    this.IDTRANSREENV = 'REENVIAR'; //Reenviar Evento
    this.IDTRANSEXCLU = 'EXCLUSAO'; //Excluir Evento
	
	// Nomes dos Eventos
    this.R1000 = 'R-1000';
    this.R1070 = 'R-1070';
    this.R2010 = 'R-2010';
    this.R2020 = 'R-2020';
    this.R2040 = 'R-2040';
    this.R2055 = 'R-2055';
    this.R2060 = 'R-2060';
    this.R2098 = 'R-2098';
    this.R2099 = 'R-2099';
    this.R5001 = 'R-5001';
    this.R5011 = 'R-5011';
    this.R9000 = 'R-9000';
    
    // Ids de ambiente
    this.ID_AMBIENTE_PRODUCAO = '1';
    this.ID_AMBIENTE_PRODRESTRITA = '2'; 
    
    //Regras exclusao
    this.idNaoPodeExcluir = ['C', 'M', 'R'];
    this.msgObsNaoExcluir = ['MS1089']; //Codigos de mesagem que significam que foram enviados e aceitos pela receita
	
	this.transmissao = function(arrEventos) {
	    var operacao;
	    
	    if (arrEventos[0].IDOPERACAO === 'C') {
	        operacao = this.IDTRANSCONSU;
	    } else if (arrEventos[0].IDOPERACAO === 'I' || arrEventos[0].IDOPERACAO === 'A' || arrEventos[0].IDOPERACAO == 'R') {
	        operacao = this.IDTRANSENVIO;
	    } else if (arrEventos[0].IDOPERACAO === 'E') {
	        operacao = this.IDTRANSEXCLU;
	    } else {
	        ScreenService.showMessage(Messages_fiscal.OEPRACAO_SEM_ROTA, 'error');
	    }
	   
        return General.callRestEngine('fis_acessorias#/setTransmissaoReinf/'+operacao,'DataSet', arrEventos);
	    
    };
    
    this.validaTransmissaoLote = function(rows){
      
        if(rows.length>100){
            ScreenService.showMessage(Messages_fiscal.TAMANHO_LOTE_ENVIO_REINF, 'error');
            return false;
        }
        
        var lotePendente      = true;
        var versaoDiferente   = false;
        var ambienteDiferente = false;
        var mesDiferente      = false;
        var eventoDiferente   = rows[0].NMEVENTO;
        
        var ambiente       = rows[0].IDAMBIENTE;
        var versao         = rows[0].DSVERSAO;
        var mesCompetencia = rows[0].DTMESCOMPETENC;
    
        
        rows.forEach(function (row) {
            if (row.IDSTATUS !== 'P' && row.IDSTATUS !== 'E') 
                lotePendente = false;
                
            /*if (row.NMEVENTO == 'R-2099' && row.IDSTATUS == 'R') 
                lotePendente = true;
            */
            if(row.IDAMBIENTE !== ambiente)
                ambienteDiferente = true;
            
            if(row.DSVERSAO !== versao)
                versaoDiferente = true;
                
            if(row.NMEVENTO !== eventoDiferente) eventoDiferente = true;
            
            if(row.DTMESCOMPETENC !== mesCompetencia)
                mesDiferente = true;
        });

        if(!lotePendente){
            ScreenService.showMessage(Messages_fiscal.LOTE_STATUS_ERROR, 'error');
            return false;
        }
        
        if(ambienteDiferente){
            ScreenService.showMessage(Messages_fiscal.LOTE_AMBIENTE_ERROR, 'error');
            return false;
        }
        
        if(eventoDiferente === true) {
            ScreenService.showMessage(Messages_fiscal.EVENTO_DIFERENTE_ERROR, 'error');
            return false;
        }
        
        if(versaoDiferente){
            ScreenService.showMessage(Messages_fiscal.LOTE_VERSAO_ERROR, 'error');
            return false;
        }
        
        if(mesDiferente){
            ScreenService.showMessage(Messages_fiscal.LOTE_COMPETENCIA_ERROR, 'error');
            return false;
        }
        
        if (rows.length != rows.filter(function(evento) {
            return evento.IDOPERACAO === rows[0].IDOPERACAO;
        }).length) {
            ScreenService.showMessage(Messages_fiscal.OPERACAO_DIFERENTE, 'error');
            return false;
        }
        return true;
    };
    
    this.verificarPodeExcluir = function(evento) {
        /*if (evento.IDAMBIENTE == this.ID_AMBIENTE_PRODRESTRITA) {
            return true;
        } else*/ 
        if ((evento.IDSTATUS == 'E' && evento.NRRECIBO) || (evento.IDSTATUS == 'R')){
             return false;
        } else if (evento.IDSTATUS == 'E' && !evento.NRRECIBO) {
            //Cairá nessa condição se um evento for enviado e aceito pela receita, 
            //porém, por instabilidade de algum servidor o recibo não seja gravado,
            //dessa forma o evento não pode ser excluído, pois ID não pode ser
            //perdido. A variável msgObsNaoExcluir armazena os codigos de mensagens
            //que identificam esses problemas. Esses códigos podem ser acrescidos com
            //o tempo, caso identique outros erros que caem na mesma situação, ou 
            //situação parecida.
            return this.vericiaObsParaExcluir(evento.DSOBSERVACAO);
        } else {
            return this.idNaoPodeExcluir.indexOf(evento.IDSTATUS) == -1;
        }    
    };
    
    this.vericiaObsParaExcluir = function(message) {
        //Se existir algum código, que identifica que o evento não pode ser exclído,
        //dentro do valor da observação daquele evento, a função retornará false,
        //caso não exista rertornará true;
        return this.msgObsNaoExcluir.filter(function(codErro) {
            return message && message.indexOf(codErro) != -1;
        }).length == 0;
    };
  
};

Configuration(function(ContextRegister) {
     ContextRegister.register('Fis75000_Efd_Reinf_Service', Fis75000_Efd_Reinf_Service);
});
